#  Creating starting from tempo.g 
#/*
# * Tempo grammar: for dparser
# *
# * Based on Appendix C of the Tempo User Guide
# *
# * This should be enough to parse all the examples in Tempo/examples/newUserGuide
# * (except twoTaskRace.tioa because it includes a simulation relation)
# */
#
#/*
# * Top-level:
# * - removed 'simDef and simProg'
# * - simplified the invariant syntax
# */
# ASHISH: Important changes to grammar
# 1. Removing ID [ typeList ] from typeRef
#    because it conflicts with Enumeration [ ] type

from dparser import Parser
import sys
import os
from xml.dom.minidom import getDOMImplementation, parseString
import xmlHelpers
import HSalXMLPP
from xmlHelpers import createNodeInfixApp, createNodeApp, createNodeTag, createNodeTagChildn, createNodeTagChild, createNodeTagChild2, createNodeTagChild3, createNodeAnd

typemap = { 'Int':'INTEGER', 'Bool':'BOOLEAN', 'Nat':'NATURAL', 'DiscreteReal':'REAL', 'Real':'REAL', 'AugmentedReal':'REAL' }
currN = 0

def d_spec(s, nodes):
    "spec : ( typeDecls | imports | funDecls | vocabDef | autoDef | invDef )+ "
    ans = []
    for i in s[0]:
        if isinstance(i[0], list):
            ans.extend(i[0])	# typeDecls, vocabDef will return list
        else:
            ans.append(i[0])	# autoDef return MODULEDECL
    ans = filter(None, ans)
    return createNodeTagChildn('CONTEXTBODY', ans)

def d_imports(s, nodes):
    "imports : 'imports' vocabRef (',' vocabRef )* "
    print 'Warning: Imports are always being assumed; may cause potential name clash'
    return []

def d_vocabRef(s, nodes): 
    "vocabRef : ID ( '(' actual (',' actual )* ')' )? "

def d_funDecls(s, nodes): 
    "funDecls : 'let' (funDecl ';')+ "

def d_funDecl(s, nodes): 
    "funDecl : ID '(' (ID (',' ID)* )? ')' ':' typeSignature '=' expr "

def d_invDef(s, nodes): 
    "invDef : 'invariant' idOrNumeral? 'of' ID ':' expr ';' "
    # return ASSERTIONDECLARATION MODULEMODELS
    global currN
    if len(s[1]) > 0:
        name = s[1][0]
        if name.tagName == 'NUMERAL':
            val = HSalXMLPP.valueOf(name)
            name = createNodeTag('IDENTIFIER', 'p'+val)
        else:
            name.tagName = 'IDENTIFIER'
    else:
        currN += 1
        name = createNodeTag('IDENTIFIER', 'p'+str(currN))
    form = createNodeTag('ASSERTIONFORM', 'THEOREM')
    Gnode = createNodeTag('NAMEEXPR', 'G')
    expr = createNodeApp(Gnode, [s[5]])
    modname = s[3]
    modname.tagName = 'MODULENAME'
    modactuals = createNodeTagChildn('MODULEACTUALS', [])
    modinst = createNodeTagChild2('MODULEINSTANCE', modname, modactuals)
    modmodels = createNodeTagChild2('MODULEMODELS', modinst, expr)
    return createNodeTagChild3('ASSERTIONDECLARATION', name, form, modmodels)

def d_idOrNumerals(s, nodes): 
    "idOrNumerals : idOrNumeral ( ',' idOrNumeral )* "
    ans = [s[0]]
    for i in s[1]:
        ans.append(i[1])
    for i in ans:
        i.tagName = 'SCALARELEMENT'
    return ans

def d_idOrNumeral(s, nodes): 
    "idOrNumeral : (ID | INT ) "
    return s[0][0]

def d_actual(s, nodes): 
    "actual : (expr | 'Type' typeRef ) "
    if len(s[0]) == 2:
        print 'Ignoring Type as an actual'
        return None
    else:
        return s[0][0]


# /*
#  * Type declarations
#  */
def d_typeDecls(s, nodes): 
    "typeDecls : 'types' typeDecl ( ',' typeDecl )* "
    # return list of XML nodes
    s[1].tagName = 'TYPEDECLARATION'	# change from 'VARDECL'
    ans = [ s[1] ]
    for i in s[2]:
        i[1].tagName = 'TYPEDECLARATION'
        ans.append( i[1] ) 
    return ans

def d_typeDecl(s, nodes): 
    "typeDecl : ID (':' typeRef)? "
    assert len(s[1])>0, 'Require typeref for now'
    name = s[0]
    name.tagName= 'IDENTIFIER'
    return createNodeTagChild2('VARDECL', name, s[1][0][1])

def d_typeRef(s, nodes): 
    "typeRef : ID | 'Enumeration' '[' idOrNumerals ']' | 'Tuple' '[' fieldDecls ']' | 'Union' '[' fieldDecls ']' "
    # "typeRef : ID | 'Enumeration' '[' idOrNumerals ']' | 'Tuple' '[' fieldDecls ']' | 'Union' '[' fieldDecls ']' | ID '[' typeList ']' "
    if s[0] == 'Enumeration':
        return createNodeTagChildn('SCALARTYPE', s[2])
    elif s[0] == 'Tuple':
        assert False, 'Not handling tuple type for now'
    elif s[0] == 'Union':
        assert False, 'Not handling union type for now'
    elif len(s) == 1:
        name = HSalXMLPP.valueOf(s[0])
        if typemap.has_key(name):
            return createNodeTag('TYPENAME', typemap[name])
        else:
            print 'WARNING: Unknown type {0} maybe present in translated HSal'.format(name)
            return createNodeTag('TYPENAME', name)
    else:
        assert False, 'Not handling ID [typelist] type for now'

def d_fieldDecls(s, nodes): 
    "fieldDecls : fieldDecl ( ',' fieldDecl )* "

def d_fieldDecl(s, nodes): 
    "fieldDecl : ID ( ',' ID )* ':' typeRef "

def d_typeSignature(s, nodes): 
    "typeSignature : (typeList)? '->' typeRef "

def d_typeList(s, nodes): 
    "typeList : typeRef ( ',' typeRef )* "


#/*
# * Vocabulary definition: we don't support the infix/mixfix/prefix specs
# * operator declarations must be of the form ID: typeSignature
# */
def d_vocabDef(s, nodes): 
    "vocabDef : 'vocabulary' ID (formals)? (importsAndTypes)? (operators)? 'end' "
    # map this to CONSTANTDECLARATION...
    print 'Warning: Vocabulary not separated; adding in same name space'
    ans = []
    for i in s[2]:
        print 'Warning: Not sure if vardecls can/should be in outer CONTEXT'
        ans.extend( i[0] )
    for i in s[3]:
        ans.extend( i[0] )
    return ans

def d_importsAndTypes(s, nodes): 
    "importsAndTypes : ( imports | typeDecls )+ "
    # return a list of things that can go inside CONTEXT
    ans = []
    for i in s[0]:
        ans.extend(i[0])
    return ans

def d_operators(s, nodes): 
    "operators : 'operators' opDecl (',' opDecl )* "
    print 'datatypes not handled presently'

def d_opDecl(s, nodes): 
    "opDecl : ID ( ',' ID )* ':' typeSignature "



#/*
# * Automaton definitions
# * - no support for tasks and schedules
# */
def d_autoDef(s, nodes): 
    "autoDef : 'automaton' ID ( formals (where)? )? (imports)? autoCore "
    def mkInExpr(vardecl, expr):
        lhs = createNodeTag('NAMEEXPR', HSalXMLPP.getNameTag(vardecl,'IDENTIFIER'))
        nam = lhs.cloneNode(True)
        nam.tagName = 'IDENTIFIER'
        typ = HSalXMLPP.getArg(vardecl, 2).cloneNode(True)
        setpredexpr = createNodeTagChild3('SETPREDEXPRESSION', nam, typ, expr.cloneNode(True))
        rhs = createNodeTagChild('RHSSELECTION', setpredexpr)
        return createNodeTagChild2('SIMPLEDEFINITION', lhs, rhs)
    name = s[1]
    name.tagName = 'IDENTIFIER'	# change from NAMEEXPR to IDENTIFIER
    # assert len(s[2])==0, 'not handling formals in module decl'
    if len(s[2]) > 0:
        childs = s[2][0][0]
    else:
        childs = []
    vardecls = createNodeTagChildn('VARDECLS',childs)
    if len(s[3]) != 0: 
        print 'Warning: Import declaration found; but assuming handled correctly'
    # add formals as local variables if where clause is present
    basemodule = s[4]
    if len(s[2]) > 0 and len(s[2][0]) > 1 and len(s[2][0][1]) > 0 and basemodule.tagName == 'BASEMODULE':
        ans = createNodeTagChildn('LOCALDECL',childs)
        # add ans as a child of BASEMODULE
        basemodule.insertBefore(ans, basemodule.firstChild)    
        # now add initialization block for these new variables
        expr = s[2][0][1][0]
        assgns = [ mkInExpr(i, expr) for i in childs ]
        initdecls = basemodule.getElementsByTagName('INITDECL')
        assert len(initdecls) > 0, 'Error: No initialization block found in tioa'
        initdecl = initdecls[0]
        for i in assgns: 
            initdecl.appendChild(i)
    moddecl = createNodeTagChildn('MODULEDECLARATION', [name,vardecls,basemodule])
    return moddecl

def d_formals(s, nodes): 
    "formals : '(' formal (',' formal )* ')' "
    ans = s[1]
    for i in s[2]:
        ans.extend(i[1])
    return ans

def d_formal(s, nodes): 
    "formal : ID (',' ID)* ':' (typeRef | 'Type') "
    name = s[0]
    name.tagName = 'IDENTIFIER'
    typename = s[3][0]
    if isinstance(typename,str):
        print 'WARNING: ignoring Type in formal decl'
        return []
    ans = [ createNodeTagChild2('VARDECL', name, typename) ]
    for i in s[1]:
        name = i[1]
        name.tagName = 'IDENTIFIER'
        ans.append( createNodeTagChild2('VARDECL', name, typename.cloneNode(True)) )
    return ans

def d_where(s, nodes): 
    "where : 'where' expr "
    print 'Warning: Ignoring an occurrence of where'
    return s[1]


def d_autoCore(s, nodes): 
    "autoCore : basicAutomaton | composedAutomaton "
    return s[0]

def d_basicAutomaton(s, nodes): 
    "basicAutomaton : (actionSignature)? states (funDecls)? (transitions)? (trajectories)? "
    # return BASEMODULE, childs has localdecl, inputdecl, initdecl, transdecl
    (decllist, initdecl) = s[1]
    childs = decllist
    if initdecl != None:
        childs.append(initdecl) 
    trans = []
    if len(s[3]) > 0:
        trans.extend( s[3][0][0] )
    if len(s[4]) > 0:
        trans.extend( s[4][0][0] )
    somecommands = createNodeTagChildn('SOMECOMMANDS', trans)
    tdecl = createNodeTagChild('TRANSDECL', somecommands)
    childs.append(tdecl)
    assert len(s[2])==0, 'What are funDecls in basicAutomaton?'
    return createNodeTagChildn('BASEMODULE', childs)


#/*
# * Signature
# */
def d_actionSignature(s, nodes): 
    "actionSignature : 'signature' (formalActions)+ "

def d_formalActions(s, nodes): 
    "formalActions : \
      'input' formalAction ( ',' formalAction )* \
    | 'output' formalAction ( ',' formalAction )* \
    | 'internal' formalAction ( ',' formalAction )* "

def d_formalAction(s, nodes): 
    "formalAction : ID ( '(' sigFormal (',' sigFormal )* ')' (where)? )? "

def d_sigFormal(s, nodes): 
    "sigFormal : 'const' expr | ID (',' ID )* ':' typeRef "



#/*
# * State variable declaration + initialization
## * - note: initially seems to be an old form (could remove it?)
# */
def d_states(s, nodes): 
    "states : 'states' (state ';')+ (initially)? | 'states' "
    if len(s) == 1:
        return ([], None)
    decls =  []
    assgns = []
    for i in s[1]:
        (decl,assgn) = i[0]
        decls.append(decl)
        if assgn != None:
            assgns.append(assgn)
    return (decls, createNodeTagChildn('INITDECL', assgns) )

def d_state(s, nodes): 
    "state : ID ':' typeRef ( ':=' value )? "
    name = s[0]
    varname = name.cloneNode(True)
    varname.tagName = 'IDENTIFIER'
    vdecl = createNodeTagChild2('VARDECL', varname, s[2])
    decl = createNodeTagChild('OUTPUTDECL', vdecl)
    if len(s[3]) > 0:
        rhs = createNodeTagChild('RHSEXPRESSION', s[3][0][1])
        assgn = createNodeTagChild2('SIMPLEDEFINITION', name, rhs)
    else:
        assgn = None
    return (decl, assgn)

def d_initially(s, nodes): 
    "initially : 'initially' expr ';' "
    print 'WARNING: initially expr being IGNORED, Output hsal not complete'




#/*
# * Transitions: allowed only 'var := value' and if-then-else in the 'effStmt'
# * Also removed the 'ensuring' clause and 'urgency'
# *      simplified the precondition syntax to 'pre' expr ';'
# */
def d_transitions(s, nodes): 
    "transitions : 'transitions' (transition)+ "
    # return a list of XML nodes GUARDEDCOMMAND
    ans = [ ]
    for i in s[1]:
        ans.append(i[0])
    return ans

def d_transition(s, nodes): 
    "transition : \
      'input' transitionCore  \
    | 'output' transitionCore  \
    | 'internal' transitionCore "
    return s[1]

def d_transitionCore(s, nodes): 
    "transitionCore : ID ( '(' actionActuals ')' (where)? )? (localVars)? (precondition)? (effect)? "
    # maps to GUARDEDCOMMAND
    print 'WARNING: Ignoring transition labels and events'
    assert len(s[2]) == 0, 'localVars in transitions: unable to handle now'
    if len(s[3]) > 0:
        guard = createNodeTagChild('GUARD', s[3][0][0])
    else:
        true = createNodeTag('NAMEEXPR', 'TRUE')
        guard = createNodeTagChild('GUARD', true)
    if len(s[4]) > 0:
        assignments = createNodeTagChildn('ASSIGNMENTS', s[4][0][0])
    else:
        assignments = createNodeTagChildn('ASSIGNMENTS', [])
    return createNodeTagChild2('GUARDEDCOMMAND', guard, assignments)

def d_actionActuals(s, nodes): 
    "actionActuals : expr (',' expr)* "

def d_localVars(s, nodes): 
    "localVars : 'locals' ( localDecl ';' )+ "

def d_precondition(s, nodes): 
    "precondition : 'pre' expr ';' "
    return s[1]

def d_effect(s, nodes): 
    "effect : 'eff' effProgram  "
    return s[1]

def d_localDecl(s, nodes): 
    "localDecl : ID ':' typeRef ( ':=' value ) ? "

def d_effProgram(s, nodes): 
    "effProgram : ( effStmt )+ "
    ans = [ ]
    for i in s[0]:
        ans.append( i[0] )
    return ans

# ASHISH: changing grammar of effSmt -- removing ; option
def d_effStmt(s, nodes): 
    "effStmt : lvalue ':=' value ';' \
    | 'if' effCondRec 'fi' ';' "
    # return a SIMPLEDEFINITION
    if len(s) == 4 and s[1] == ':=' :
        lhs = createNodeTagChild('NEXTOPERATOR', s[0])
        rhs = createNodeTagChild('RHSEXPRESSION', s[2])
        return createNodeTagChild2('SIMPLEDEFINITION', lhs, rhs)
    else:
        assert False, 'if-fi in effStmt not handled'

def d_effCondRec(s, nodes): 
    "effCondRec : expr 'then' effProgram ( 'elseif' effCondRec | 'else' effProgram ) ? "



#/*
# * Trajectories:
# * - simplified the syntax: 
# *   Formals and local functions are not supported. 
# *   They don't seem to be use in any of the examples anyway.
# * - invariant clause: simplified to 'invariant expr ;'
# */
def d_trajectories(s, nodes): 
    "trajectories : 'trajectories' (trajectory)+ "
    # return a list of XML nodes GUARDEDCOMMAND
    ans = []
    for i in s[1]:
        ans.append(i[0])
    return ans
    # return createNodeTagChildn('SOMECOMMANDS', s[0])

def d_trajectory(s, nodes): 
    "trajectory : 'trajdef' ID (trajInvariant)? (stopCond)? (evolve)? "
    if len(s[2]) == 0:
        guard = createNodeTag('NAMEEXPR', 'TRUE')
    else:
        guard = s[2][0][0]
    if len(s[3]) != 0:
        stopwhen = createNodeApp("NOT", [ s[3][0][0] ])
        guard = createNodeAnd([guard, stopwhen])
    guard = createNodeTagChild('GUARD', guard)
    assert len(s[4]) > 0
    assignments = s[4][0][0]
    return createNodeTagChild2('GUARDEDCOMMAND', guard, assignments)

def d_trajInvariant(s, nodes): 
    "trajInvariant : 'invariant' expr ';' "
    return s[1]

def d_stopCond(s, nodes): 
    "stopCond : 'stop' 'when' expr ';' "
    return s[2]

def d_evolve(s, nodes): 
    "evolve : 'evolve' ( evolveStmt)+  "
    childs = []
    for i in s[1]:
        childs.append(i[0])
    return createNodeTagChildn('ASSIGNMENTS', childs)

def d_evolveStmt(s, nodes): 
    "evolveStmt : 'd' '(' ID ')' '=' expr ';' "
    # "evolveStmt : 'd' '(' ID ')' '=' expr ';' | expr ';' "
    # this should map to assignment
    if len(s) == 7:
        name = HSalXMLPP.valueOf(s[2])
        newname = createNodeTag('NAMEEXPR',name+'dot')
        lhs = createNodeTagChild('NEXTOPERATOR', newname)
        rhs = createNodeTagChild('RHSEXPRESSION', s[5])
        return createNodeTagChild2('SIMPLEDEFINITION', lhs, rhs)
    else:
        print 'Complex evolveStmt not handled; l={0}, {1}'.format(len(s),s[0])

#/*
# * Composed Automaton
# */
def d_composedAutomaton(s, nodes): 
    "composedAutomaton : components (hiddenActionSets )? "
    if len(s[1]) > 0:
        print 'Warning: hiddenActionSets are not translated into HybridSal'
    # s[0] should be a list of MODULEINSTANCEs
    mod = None
    for i in s[0]:
        if mod == None:
            mod = i
        else:
            mod = createNodeTagChild2('SYNCHRONOUSCOMPOSITION', mod, i)
    return mod
   

def d_components(s, nodes): 
    "components : 'components' ( component ';' )+ "
    # return list of MODULEINSTANCEs
    ans = []
    for i in s[1]:
        ans.append(i[0])
    return ans

def d_component(s, nodes): 
    "component : ID ('[' varList (',' varList )* ']' )? (':' componentDef )? (where)? "
    # return a MODULEINSTANCE
    if len(s[2]) > 0:  # 'ERROR: Expected module instantiation'
        return s[2][0][1]
    else:
        modname = s[0]
        modname.tagName = 'MODULENAME'
        modarg = createNodeTagChildn('MODULEACTUALS', [])
        return createNodeTagChild2('MODULEINSTANCE', modname, modarg)

def d_componentDef(s, nodes): 
    "componentDef : ID ( '(' actual (',' actual )* ')' )? "
    modname = s[0]
    modname.tagName = 'MODULENAME'	# change from NAMEEXPR
    args = []
    if len(s[1]) > 0:
        args.append(s[1][0][1])
        if isinstance(s[1][0][2], list):
            for i in s[1][0][2]:
                args.append(i[1])
    if args != []:
        modactuals = createNodeTagChildn('TUPLELITERAL', args)
    else:
        modactuals = []
    modarg = createNodeTagChild('MODULEACTUALS', modactuals)
    return createNodeTagChild2('MODULEINSTANCE', modname, modarg)

def d_hiddenActionSets(s, nodes): 
    "hiddenActionSets : 'hidden' ( actionSet ';' ) + "

def d_actionSet(s, nodes): 
    "actionSet : ( compInstance '.')? ID ('(' expr (',' expr )* ')' )? "

def d_compInstance(s, nodes): 
    "compInstance : ID ('[' expr (',' expr )* ']' )? ( '.' ( '[' expr (',' expr)* ']' )? )* "

def d_varLis(s, nodes):
    "varList: ID (',' ID)* ':' typeRef "


#/*
# * Expressions
# */
def d_expr(s, nodes): 
    "expr : \
      'if' expr 'then' expr 'else' expr \
    | expr BINARY_OP expr \
    | UNARY_OP expr \
    | QUANTIFIER ID ':' typeRef expr \
    | '[' expr (',' expr)* ']' \
    | '{' ID ':' typeRef 'where' expr '}' \
    | '{' expr (',' expr)* '}' \
    | expr ( '.' ID | '[' expr (',' expr)* ']' )+ \
    | ID '(' expr (',' expr)* ')' \
    | '(' typeRef ')' expr \
    | 'true' \
    | 'false' \
    | '\infty' \
    | '{}' \
    | INT \
    | FLOAT \
    | ID \
    | ID '\\'' \
    | '(' expr ')' "
    # return EXPRESSION
    if len(s) == 6 and s[0] == 'if':  # if-then-else expression
        assert False, 'Not handling if-then-else right now'
    elif len(s) == 5 and s[2] == ':':  # quantifier expr
        assert False, 'Not handling quantified expressions'
    elif len(s) == 4 and s[0] == '[':  # quantifier expr
        assert False, 'Not handling [ expr, expr ] whats its semantics?'
    elif len(s) == 4 and s[0] == '{':  # quantifier expr
        assert False, 'Not handling { expr, expr } whats its semantics?'
    elif len(s) == 7 and s[0] == '{':  # where expr
        assert False, 'Not handling { .. where .. } expressions'
    elif len(s) == 2 and isinstance(s[1], list):  # array/record access
        assert False, 'Not handling array/record access expressions'
    elif len(s) == 5 and s[1] == '(':  # function call
        args = [ s[2] ]
        for i in s[3]:
            args.append(i[1])
        return createNodeApp(s[0], args)
    elif len(s) == 3 and s[0] == '(':
        return s[1]
    elif len(s) == 3:  # BINARY_OP
        return createNodeInfixApp(s[1], s[0], s[2])
    elif len(s) == 2:  # UNARY_OP
        return createNodeApp(s[0], [ s[1] ])
    elif len(s) == 1 and s[0] == 'true':
        return createNodeTag('NAMEEXPR', 'TRUE')
    elif len(s) == 1 and s[0] == 'false':
        return createNodeTag('NAMEEXPR', 'FALSE')
    elif len(s) == 1 and isinstance(s[0], str) and s[0] == '\infty':
        print 'Warning: \infty in tioa model replaced by 10000 in hsal model'
        return createNodeTag('NUMERAL', '10000')
    elif len(s) == 1 and isinstance(s[0], str):
        assert False, '{0} expressions not handled'.format(s[0])
    elif len(s) == 1:  # ID , FLOAT, INT
        return s[0]
    elif len(s) == 4 and s[0] == '(':  # typeref 
        print 'ignoring typeref in expr'
        return s[3]



#/*
# * Assignment
# */
def d_lvalue(s, nodes): 
    "lvalue : ID ( '[' expr (',' expr)* ']' | '.' ID )* "
    # return NAMEEXPR
    assert len(s[1]) == 0, 'lvalue is an ARRAY access; missing code'
    return s[0]

def d_value(s, nodes): 
    "value : expr | choice "
    return s[0]

def d_choice(s, nodes): 
    "choice : 'choose' variable 'where' expr "
    assert False, 'choice operator not handled, missing code'

def d_variable(s, nodes): 
    "variable : ID (':' typeRef)? "
    return s[0]


#/*
# * TERMINALS
# */
def d_ID(s, nodes): 
    'ID : "[a-zA-Z][a-zA-Z_0-9\$]*" '
    return createNodeTag('NAMEEXPR', s[0])

def d_INT(s, nodes): 
    'INT : "[\+\-]?[0-9]+" '
    return createNodeTag('NUMERAL', s[0])

def d_FLOAT(s, nodes): 
    'FLOAT : "[\+\-]?[0-9]+\.[0-9]+" \
  | "[\+\-]?[0-9]+(\.[0-9]+)? [Ee][\+\-][0-9]+" '
    return createNodeTag('NUMERAL', s[0])

def d_QUANTIFIER(s, nodes): 
    "QUANTIFIER : '\A' | '\E' "
    assert False, 'Quantifier not handled, missing code'

def d_UNARY_OP(s, nodes): 
    "UNARY_OP : \
   '+'   $unary_op_right 40 | \
   '-'   $unary_op_right 40 | \
   '~'   $unary_op_right 40 "
    if s[0] == '~':
        op = 'NOT'
    else:
        op = s[0]
    return createNodeTag('NAMEEXPR', op)

def d_BINARY_OP(s, nodes): 
    "BINARY_OP : \
   '**'        $binary_op_left 30 | \
   '/'         $binary_op_left 29 | \
   '*'         $binary_op_left 29 | \
   '+'         $binary_op_left 28 | \
   '-'         $binary_op_left 28 | \
   '\I'        $binary_op_left 27 | \
   '\U'        $binary_op_left 26 | \
   '|-'        $binary_op_left 25 | \
   '-|'        $binary_op_left 25 | \
   '||'        $binary_op_left 24 | \
   '<'         $binary_op_left 20 | \
   '>'         $binary_op_left 20 | \
   '<='        $binary_op_left 20 | \
   '>='        $binary_op_left 20 | \
   '\notin'    $binary_op_left 20 | \
   '\in'       $binary_op_left 20 | \
   '\subset'   $binary_op_left 20 | \
   '\subseteq' $binary_op_left 20 | \
   '\supset'   $binary_op_left 20 | \
   '\supseteq' $binary_op_left 20 | \
   '='         $binary_op_left 19 | \
   '~='        $binary_op_left 19 | \
   '/\\x5c'     $binary_op_left 6 | \
   '\/'        $binary_op_left 5 | \
   '=>'        $binary_op_left 4 | \
   '<=>'       $binary_op_left 3 "
    op = s[0]
    if op == '\/':
        op = 'OR'
    elif op == '/\\':
        op = 'AND'
    elif op == '~=':
        op = '/='
    elif op == '\in':
        op = 'IN'
    return createNodeTag('NAMEEXPR', op)


def d_whitespace(s, nodes): 
    'whitespace : ( "[ \t\n\r\v\f]" | comment )* '

def d_comment(s, nodes): 
    'comment :  "%" "[^\n]*" "\n"  '

def printUsage():
    print """
NAME: dtempo - termpo parser 
SYNOPSIS: dtempo <filename.tioa>
"""
    sys.exit()

def main_aux(filename):
    global dom		# for creating the XML of the translated hybridSal
    # if .xml exists and is newer than .hioa then quit
    basename,ext = os.path.splitext(filename)
    xmlfilename = basename + ".hxml"
    #if os.path.isfile(xmlfilename):
        #if os.stat(xmlfilename).st_mtime > os.stat(filename).st_mtime:
            #print 'tempo2hsal: XML file is up to date.'
            #return xmlfilename
    # need to create .xml file
    impl = getDOMImplementation()
    dom = impl.createDocument(None, "hybridsal", None)
    xmlHelpers.setDom(dom)
    with open(filename, 'r') as f:
        x = f.read()
    y = Parser().parse(x)    # y is now a dparser.ParsedStructure instance
    body = y.getStructure()
    #print z
    (head, tail) = os.path.split(basename)
    name = createNodeTag('IDENTIFIER', tail)
    params = createNodeTagChildn('PARAMETERS',[])
    hsal = createNodeTagChild3('CONTEXT', name, params, body)
    with open(xmlfilename, 'w') as f:
        #print >> f, hsal.toprettyxml()
        print >> f, hsal.toxml()
    hsalfilename = basename + ".hsal"
    with open(hsalfilename, 'w') as f:
        HSalXMLPP.HSalPPContext(hsal, filepointer=f)
    return xmlfilename

def main():
    args = sys.argv[1:]
    if len(args) < 1 or len(args) > 1 or args[0][0] == '-':
        printUsage()
        return 1
    filename = args[0]
    if not(os.path.isfile(filename)):
        print "File does not exist. Quitting."
        return 1
    output_filename = main_aux(filename)
    return output_filename


if __name__ == '__main__':
    main()
