#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <assert.h>

#include <dparse.h>

extern D_ParserTables parser_tables_gram;

extern int d_verbose_level;


/*
 * Open filename and read all its content
 * - return a malloc'ed buffer that contains the file
 *   store the file size in *len
 * - print an error and return NULL if something fails
 */
static char* read_file(const char *filename, size_t *len) {
  FILE *f;
  char *buffer;
  long l;
  size_t file_size, n;

  buffer = NULL;
  f = fopen(filename, "r");
  if (f == NULL) {
    fprintf(stderr, "Couldn't open %s\n", filename);
    perror(filename);
    goto exit;
  }

  if (fseek(f, 0, SEEK_END)) {
    perror("fseek failed");
    goto done;
  }

  l = ftell(f);
  if (l < 0) {
    perror("ftell failed");
    goto done;
  }

  file_size = (size_t) l;
  *len = file_size;

  buffer = malloc(file_size);
  if (buffer == NULL) {
    fprintf(stderr, "could not allocate a buffer to read the file (need %u bytes)\n", (unsigned) file_size);
    goto done;
  }

  // read the file
  rewind(f);
  n = fread(buffer, file_size, 1, f);
  if (n < file_size) {
    if (ferror(f)) {
      perror("fread failed");
      free(buffer);
      goto done;
    }
    if (feof(f)) {
      fprintf(stderr, "Warning: may not have read the full file (expected: %u bytes, read %u bytes)\n", (unsigned) file_size, (unsigned) n);
      // keep going anyway
    }
  }
  
  
 done:
  fclose(f);

 exit:
  return buffer;
}



int main(int argc, char *argv[]) {
  D_Parser *parser;
  D_ParseNode *node;
  size_t length;
  char *buffer;
  int exit_code;

  if (argc != 2) {
    fprintf(stderr, "Usage: %s <filename>\n", argv[0]);
    return 1;
  }

  buffer = read_file(argv[1], &length);
  if (buffer == NULL) {
    return 1;
  }

  if (length > (size_t) INT_MAX) {
    fprintf(stderr, "File too large for D Parser\n");
    exit_code = 1; 
  } else {
    d_verbose_level = 1;
    parser = new_D_Parser(&parser_tables_gram, 0);
    node = dparse(parser, buffer, length);

    if (parser->syntax_errors) {
      fprintf(stderr, "Syntax error\n");    
      exit_code = 1;
    } else {
      printf("No syntax error detected\n");
      exit_code = 0;
    }
  }

  free(buffer);

  return exit_code;  
}
